/*
 * This file is part of libcreds.
 *
 * Copyright (C) 2009 Nokia Corporation. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <ctype.h>
#include <sys/creds.h>
#include <sys/capability.h>

/*
 * A simple test program to check the basic library functionality
 */
int main(int argc, char **argv)
{
	do {
		creds_value_t value;
		pid_t pid = --argc > 0 ? atoi(argv[argc]) : 0;
		if (argc == 0 || isdigit(*argv[argc])) {
			/* Assume numeric arguments are pid's */
			
			int index;
			creds_type_t type;
			creds_t creds = creds_gettask(pid);
			if (creds == NULL) {
				warn("Could not get credentials for pid %d", pid);
				continue;
			}
			printf("\nCredentials of task %d\n", pid);
			for (index = 0; (type = creds_list(creds, index, &value)) != CREDS_BAD; ++index) {
				char *txt;
				switch (type)
				{
				case CREDS_UID:
					printf("Uid: %d\n", (int)value);
					break;
				case CREDS_GID:
					printf("Gid: %d\n", (int)value);
					break;
				case CREDS_CAP:
					txt = cap_to_name(value);
					printf("Cap: %d (%s)\n", (int)value, txt);
					cap_free(txt);
					break;
				default:
					printf("Unknown %d: %d\n", type, (int)value);
					break;
				}
			}
			creds_free(creds);
		} else {
			/* Assume symbolic credential name, and translate to value */
			switch (creds_str2creds(argv[argc], &value)) {
			case CREDS_BAD:
				printf("%s does not translate to credentials value\n", argv[argc]);
				break;
			case CREDS_UID:
				printf("%s is uid=%u\n", argv[argc], (int)value);
				break;
			case CREDS_GID:
				printf("%s is gid=%u\n", argv[argc], (int)value);
				break;
			case CREDS_CAP:
				printf("%s is cap=%u\n", argv[argc], (int)value);
				break;
			default:
				printf("%s returns uknown type!\n", argv[argc]);
				break;
			}
		}
	} while (argc > 1);
	exit(EXIT_SUCCESS);
}
